/*
   Client pour invoquer le web service skybotresolver 
   Info: http://vo.imcce.fr/webservices/skybot
*/

#include <string.h>
#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>

#include "soapH.h"
#include "SkyBoTBinding.nsmap"

double date_iso_to_jd (char*);

int main(int argc, char *argv[])
{
   /* declaration des variables */
   const char *wsname = "http://www.imcce.fr/webservices/skybot/skybot.php";
   struct soap soap;
   struct ns1__skybotresolverRequest input;
   char *result, *substr;
   const char *lex = ";";

   char *s_options = "hi:j:n:m:o:g:";
   static struct option long_options[] = {
      {"help",     no_argument,       0, 'h'},
      {"iso",      required_argument, 0, 'i'},
      {"jd",       required_argument, 0, 'j'},
      {"name",     required_argument, 0, 'n'},
      {"mime",     required_argument, 0, 'm'},
      {"output",   required_argument, 0, 'o'},
      {"observer", required_argument, 0, 'g'}
   };
   int c, option_index = 0;
   char *s_date_iso = NULL;
   char *s_jd = NULL;
   char *s_name = NULL;

   /* initialisations par defaut */
   input.mime = malloc(strlen("text") + 1);
   strcpy(input.mime, "text");
   input.output = malloc(strlen("basic") + 1);
   strcpy(input.output, "basic");
   input.observer = malloc(strlen("500") + 1);
   strcpy(input.observer, "500");

   /* lecture des arguments */
   while((c = getopt_long(argc, argv, s_options, long_options, &option_index)) != -1)
   {
      switch (c)
      {
      case 'h':
         usage_help(argv[0], 1);
         exit(EXIT_SUCCESS);
      case 'i':
         s_date_iso = optarg;
         break;
      case 'j':
         s_jd = optarg;
         break;
      case 'n':
         s_name = optarg;
         break;
      case 'm':
         input.mime = malloc(strlen(optarg) + 1);
         strcpy(input.mime, optarg);
         break;
      case 'o':
         input.output = malloc(strlen(optarg) + 1);
         strcpy(input.output, optarg);
         break;
      case 'g':
         input.observer = malloc(strlen(optarg) + 1);
         strcpy(input.observer, optarg);
         break;
      default:
         usage_help(argv[0], 1);
         exit(EXIT_SUCCESS);
      }
   }

   if(s_jd != NULL || s_date_iso != NULL)
   {
      if(s_jd != NULL) input.epoch = atof(s_jd);
      if(s_date_iso != NULL) input.epoch = date_iso_to_jd(s_date_iso);
   }
   else
   {
      usage_help(argv[0],1);
      exit(EXIT_SUCCESS);
   }

   if(s_name != NULL)
   {
      input.name = malloc(strlen(s_name) + 1);
      strcpy(input.name, s_name);
   }
   else
   {
      usage_help(argv[0],1);
      exit(EXIT_SUCCESS);
   }

   /* Invocation du web service */
   soap_init(&soap);
   soap_call_ns1__skybotresolver(&soap, wsname, NULL, &input, &result);

   if(soap.error)
   {
      /* Print SOAP error */
      soap_print_fault(&soap, stderr);
   }
   else
   {
      /* Print result */
      substr = strtok(result, lex);
      while( substr != NULL )
      {
         printf("%s\n", substr);
         substr = strtok(NULL, lex);
      }
   }

   soap_end(&soap);
   soap_done(&soap);
   return EXIT_SUCCESS;
}
